GENCCODE_ASSEMBLY_TYPE=-a gcc-darwin
SO=dylib
SOBJ=dylib
A=a
LIBPREFIX=lib
LIB_EXT_ORDER=.68.1.dylib
COMPILE=x86_64-apple-darwin13.4.0-clang -DU_ATTRIBUTE_DEPRECATED= -D_FORTIFY_SOURCE=2 -mmacosx-version-min=10.9 -isystem /opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/include   -DU_HAVE_STRTOD_L=1 -DU_HAVE_XLOCALE_H=1 -DU_HAVE_STRING_VIEW=1  -march=core2 -mtune=haswell -mssse3 -ftree-vectorize -fPIC -fPIE -fstack-protector-strong -O2 -pipe -isystem /opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/include -fdebug-prefix-map=/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/work=/usr/local/src/conda/icu-68.1 -fdebug-prefix-map=/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p=/usr/local/src/conda-prefix -std=c11 -Wall -pedantic -Wshadow -Wpointer-arith -Wmissing-prototypes -Wwrite-strings   -fno-common -c
LIBFLAGS=-I/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/include  -dynamic
GENLIB=x86_64-apple-darwin13.4.0-clang -dynamiclib -dynamic -march=core2 -mtune=haswell -mssse3 -ftree-vectorize -fPIC -fPIE -fstack-protector-strong -O2 -pipe -isystem /opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/include -fdebug-prefix-map=/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/work=/usr/local/src/conda/icu-68.1 -fdebug-prefix-map=/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p=/usr/local/src/conda-prefix -std=c11 -Wall -pedantic -Wshadow -Wpointer-arith -Wmissing-prototypes -Wwrite-strings   -Wl,-pie -Wl,-headerpad_max_install_names -Wl,-dead_strip_dylibs -Wl,-rpath,/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/lib -L/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/lib -Wl,-rpath,/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/lib 
LDICUDTFLAGS=
LD_SONAME=-Wl,-compatibility_version -Wl,68 -Wl,-current_version -Wl,68.1 -install_name /opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_p/lib/
RPATH_FLAGS=
BIR_LDFLAGS=
AR=/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_build_env/bin/x86_64-apple-darwin13.4.0-ar
ARFLAGS=r -c
RANLIB=/opt/concourse/worker/volumes/live/6a1de213-cc5a-4fb4-61fe-d0c927733cbb/volume/icu_1611658310255/_build_env/bin/x86_64-apple-darwin13.4.0-ranlib
INSTALL_CMD=/usr/bin/install -c
